using System.Threading.Tasks;
using ExBrowserCore.Runtime.Scripts;
using ExPreloader.Runtime;
using ExTools.Helpers;
using GeoAccess.Runtime;
using Internet.Runtime;
using Permissions.Runtime;
using Services.Runtime;
using UnityEngine;
using VersionAccess.Runtime;

public class PreLoaderScope : MonoScope
    {
        [SerializeField] string _nativeSceneName;
        [SerializeField] string _webSceneConfigPath;
    
        SceneHelper _sceneHelper;
        PingInternetChecker _internetChecker;

        public override void Register() { }

        public override async Task Resolve()
        {
            Debug.Log("PreLoaderInstaller InitializeDependencies");
            
            _sceneHelper = Container.Get<SceneHelper>();
            var assembliesHelper = Container.Get<AssembliesHelper>();
        
            var permissionsManager = new PermissionsManager();
            var a = assembliesHelper.FindAllClassesWithInterface(typeof(IPermissionRequester));
            await permissionsManager.RequestAsync(a);
            
            _internetChecker = new PingInternetChecker();
            var connectionType = await _internetChecker.CheckAsync();
        
            Debug.Log($"Connection type: {connectionType}");

            if (connectionType == ConnectionType.Present)
            {
                var servicesInitializer = new ServicesInitializer();
                var b = assembliesHelper.FindAllClassesWithInterface(typeof(IServiceInitializer));
                await servicesInitializer.InitAsync(b);
            
                LoadData();
            }
            else
            {
                _sceneHelper.TryLoadScene(_nativeSceneName);
            }
        }
    
        void LoadData()
        {
            Debug.Log("PreLoaderInstaller LoadData");

            var userIpData = new UserIpData();
            userIpData.LoadUserInfo();
        
            var country = userIpData.GetCountryCode();
            var isProxy = userIpData.IsProxy();
        
            Debug.Log($"Country: {country}");
            Debug.Log($"ProxyUsed: {isProxy}");
        
            var webModel = Container.Get<WebSceneModel>();
            var geoAccessController = new GeoAccessController(webModel.GeoAccess);
            var versionAccessController = new VersionController(webModel.VersionAccess);
      
            var assert = new ExAssert();
            assert.IsStringNotEmpty(webModel.Url);
            assert.IsTrue(geoAccessController.IsProxyAllowed(isProxy));
            assert.IsTrue(geoAccessController.IsCountryAllowed(country));
            assert.IsTrue(versionAccessController.IsAllowed());

            _sceneHelper.TryLoadScene(assert.CheckAll() ? "WebScene" : _nativeSceneName);
        }
    }